@@ -19,7 +19,7 @@ def listen_properties_changed(bus, loop, callback):
         callback,
         interface=PROPERTIES_INTERFACE,
         member="PropertiesChanged",
-        path_namespace="/org/bluez"
+        path_namespace="/org/bluez",
     ).asFuture(loop)
 
 
@@ -39,7 +39,7 @@ def listen_interfaces_added(bus, loop, callback):
         callback,
         interface=OBJECT_MANAGER_INTERFACE,
         member="InterfacesAdded",
-        path_namespace="/org/bluez"
+        path_namespace="/org/bluez",
     ).asFuture(loop)
 
 
@@ -59,5 +59,5 @@ def listen_interfaces_removed(bus, loop, callback):
         callback,
         interface=OBJECT_MANAGER_INTERFACE,
         member="InterfacesRemoved",
-        path_namespace="/org/bluez"
+        path_namespace="/org/bluez",
     ).asFuture(loop)
