@@ -11,14 +11,9 @@ import pytest
 _IS_CI = os.environ.get("CI", "false").lower() == "true"
 _IS_AZURE_PIPELINES = os.environ.get("SYSTEM_HOSTTYPE", "") == "build"
 
-if platform.system() != "Darwin":
-    _OS = os.environ.get("AGENT_OS").lower()
-else:
-    _OS = platform.system()
-
 
 @pytest.mark.skipif(
-    condition=(_IS_CI or _IS_AZURE_PIPELINES) and (_OS in ("linux", "darwin")),
+    condition=_IS_AZURE_PIPELINES and (platform.system().lower() in ("linux", "darwin")),
     reason="""Cannot run on Azure Pipelines with
     Ubuntu 16.04 or macOS installed.""",
 )
